@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 2002 Double Precision, Inc.
+** Copyright 1998 - 2003 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
@@ -31,6 +31,9 @@
 
 #include	"maildirquota.h"
 #include	"maildirmisc.h"
+#include	"maildircreate.h"
+#include	"quotawarnmsg.h"
+#include	"../rfc822/rfc822.h"
 #include	<stdio.h>
 #include	<stdlib.h>
 #include	<string.h>
@@ -44,7 +47,7 @@
 #include	<time.h>
 #include	<numlib/numlib.h>
 
-static const char rcsid[]="$Id: maildirquota.c,v 1.13 2002/06/13 05:18:35 mrsam Exp $";
+static const char rcsid[]="$Id: maildirquota.c,v 1.21 2003/02/13 03:19:45 mrsam Exp $";
 
 static void parsequotastr(const char *, struct maildirquota *);
 
@@ -532,7 +535,7 @@ static int doaddquota(struct maildirsize *info, int maildirsize_fd,
 			b[0]=MDQUOTA_SIZE;
 			b[1]=0;
 
-			strcat(strcat(buf, str_off_t(info->quota.nbytes, n)),
+			strcat(strcat(buf, libmail_str_off_t(info->quota.nbytes, n)),
 			       b);
 		}
 
@@ -547,17 +550,17 @@ static int doaddquota(struct maildirsize *info, int maildirsize_fd,
 				strcat(buf, ",");
 
 			strcat(strcat(buf,
-				      str_size_t(info->quota.nmessages, n)),
+				      libmail_str_size_t(info->quota.nmessages, n)),
 			       b);
 		}
 		strcat(buf, "\n");
 	}
 
 	sprintf(buf + strlen(buf),
-		"%8s ", str_off_t(maildirsize_size, n));
+		"%8s ", libmail_str_off_t(maildirsize_size, n));
 
 	sprintf(buf + strlen(buf),
-		"%4s\n", str_off_t(maildirsize_cnt, n));
+		"%4s\n", libmail_str_off_t(maildirsize_cnt, n));
 
 	p=buf;
 	cnt=strlen(buf);
@@ -600,8 +603,8 @@ char	*p;
 		time(&t);
 		strcat(strcpy(p, dir), "/tmp/");
 		sprintf(p+strlen(p), "%s.%s_NeWmAiLdIrSiZe.%s",
-			str_time_t(t, tbuf),
-			str_pid_t(getpid(), pbuf), hostname);
+			libmail_str_time_t(t, tbuf),
+			libmail_str_pid_t(getpid(), pbuf), hostname);
 
 		if (stat( (const char *)p, &stat_buf) < 0 &&
 			(*fd=maildir_safeopen(p,
@@ -769,7 +772,7 @@ int maildirquota_countfile(const char *n)
 
 	for ( ; *n; n++)
 	{
-		if (n[0] != ':' || n[1] != '2' ||
+		if (n[0] != MDIRSEP[0] || n[1] != '2' ||
 		    n[2] != ',')	continue;
 		n += 3;
 		while (*n >= 'A' && *n <= 'Z')
@@ -864,6 +867,17 @@ void maildir_quota_deleted(const char *maildir,
 	maildir_closequotafile(&info);
 }
 
+void maildir_quota_recalculate(const char *maildir)
+{
+	struct maildirsize info;
+
+	if (maildir_openquotafile(&info, maildir))
+		return;
+	info.recalculation_needed=1;
+
+	maildir_readquota(&info);
+	maildir_closequotafile(&info);
+}
 
 int maildir_quota_delundel_start(const char *maildir,
 				 struct maildirsize *info,
@@ -886,12 +900,12 @@ void maildir_quota_delundel_end(struct maildirsize *info,
 				long msgsize, int nmsgs)
 {
 #if TRASHQUOTA
-	return (0);
+	return;
 #else
 	if (nmsgs < 0)
 		return;
 
-	return maildir_quota_add_end(info, msgsize, nmsgs);
+	maildir_quota_add_end(info, msgsize, nmsgs);
 #endif
 }
 
@@ -906,3 +920,161 @@ void maildir_quota_set(const char *dir, const char *quota)
 		maildir_closequotafile(&info);
 	}
 }
+
+
+static void do_deliver_warning(const char *msgfile, const char *dir)
+{
+int	fdin, fd;
+FILE *fpout;
+time_t	t;
+size_t	l, msg_len;
+char	*qname = 0;
+struct stat	sb;
+char	hostname[256];
+char	buf[4096];
+size_t	n;
+struct	maildirsize info;
+struct maildir_tmpcreate_info createInfo;
+
+	if ((fdin=open(msgfile, O_RDONLY)) < 0)
+		return;
+
+	l = strlen(dir)+sizeof("/quotawarn");
+
+	/* Send only one warning every 24 hours */
+	if ((qname = malloc(l)) == 0)
+	{
+		close(fdin);
+		return;
+	}
+
+	strcat(strcpy(qname, dir), "/quotawarn");
+	time(&t);
+	if (stat(qname, &sb) == 0 && ((sb.st_mtime + 86400) > t))
+	{
+		free(qname);
+		close(fdin);
+		return;
+	}
+
+	fd = open(qname, O_WRONLY|O_CREAT|O_TRUNC, 0644);
+	free(qname);
+	if (!fd)
+	{
+		close(fdin);
+		return;
+	}
+	write(fd, buf, 0);
+	close(fd);
+
+	strcpy(buf, "Date: ");
+	rfc822_mkdate_buf(t, buf+strlen(buf));
+	strcat(buf, "\n");
+
+	hostname[0]=0;
+	hostname[sizeof(hostname)-1]=0;
+	gethostname(hostname, sizeof(hostname)-1);
+	sprintf(buf, "%sMessage-Id: <%lu.overquota@%-1.256s>\n",
+		buf, (unsigned long)t, hostname);
+
+	if (stat(msgfile, &sb) < 0) {
+		close(fdin);
+		return;
+	}
+	msg_len=strlen(buf)+sb.st_size;
+
+
+
+	maildir_tmpcreate_init(&createInfo);
+	createInfo.maildir=dir;
+	createInfo.uniq="warn";
+	createInfo.msgsize=msg_len;
+	createInfo.doordie=1;
+
+	if ((fpout=maildir_tmpcreate_fp(&createInfo)) == NULL)
+	{
+		close(fdin);
+		return;
+	}
+
+	fprintf(fpout, "%s", buf);
+
+	while ((n=read(fdin, buf, sizeof(buf))) > 0)
+	{
+		fwrite(buf, n, 1, fpout);
+	}
+	close(fdin);
+
+	if (fflush(fpout) || ferror(fpout))
+	{
+		fclose(fpout);
+		unlink(createInfo.tmpname);
+		maildir_tmpcreate_free(&createInfo);
+		return;
+	}
+
+	if (fseek(fpout, 0L, SEEK_SET) >= 0)
+	{
+		/* Make sure the quota message's size itself is factored into
+		** the quota. Deliver the message regardless of whether the
+		** user is over quota.
+		*/
+		if (maildirquota_countfolder(dir))
+		{
+			maildir_quota_add_start(dir, &info, msg_len, 1, NULL);
+			maildir_quota_add_end(&info, msg_len, 1);
+		}
+	}
+
+	fclose(fpout);
+	if (maildir_movetmpnew(createInfo.tmpname,
+			       createInfo.newname))
+	{
+		unlink(createInfo.tmpname);
+		maildir_tmpcreate_free(&createInfo);
+		return;
+	}
+	maildir_tmpcreate_free(&createInfo);
+}
+
+void maildir_deliver_quota_warning(const char *dir, const int percent)
+{
+	size_t l;
+	char *p;
+	struct stat	sb;
+
+	/* If we delivered to a folder, dump the warning message into INBOX */
+
+	l = strlen(dir)+sizeof("/maildirfolder");
+	if ((p = malloc(l)) == 0)
+		return;
+
+	strcat(strcpy(p, dir), "/maildirfolder");
+
+	/* If delivering to a folder, find quotawarn in its parent directory */
+
+	if (stat(p, &sb) == 0)
+	{
+		strcat(strcpy(p, dir), "/..");
+		maildir_deliver_quota_warning(p, percent);
+		free(p);
+		return;
+	}
+	free(p);
+
+	if (percent >= 0)
+	{
+		struct maildirsize info;
+
+		if (maildir_openquotafile(&info, dir) == 0)
+		{
+			if (maildir_readquota(&info) >= percent)
+			{
+				maildir_closequotafile(&info);
+				do_deliver_warning(QUOTAWARNMSG, dir);
+				return;
+			}
+			maildir_closequotafile(&info);
+		}
+	}
+}
