@@ -2,7 +2,7 @@
 #define	maildirquota_h
 
 /*
-** Copyright 1998 - 2002 Double Precision, Inc.
+** Copyright 1998 - 2003 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
@@ -20,7 +20,7 @@
 extern "C" {
 #endif
 
-static const char maildirquota_h_rcsid[]="$Id: maildirquota.h,v 1.5 2002/06/13 05:18:35 mrsam Exp $";
+static const char maildirquota_h_rcsid[]="$Id: maildirquota.h,v 1.7 2003/02/13 03:19:46 mrsam Exp $";
 
 #define	MDQUOTA_SIZE	'S'	/* Total size of all messages in maildir */
 #define	MDQUOTA_BLOCKS	'B'	/* Total # of blocks for all messages in
@@ -127,6 +127,10 @@ void maildir_quota_delundel_end(struct maildirsize *info,
 
 void maildir_quota_set(const char *dir, const char *quota);
 
+	/* Forcibly recalculate the maildir's quota */
+
+void maildir_quota_recalculate(const char *maildir);
+
 
 	/*
 	** Should the following folder/file be included in the quota?
@@ -136,6 +140,8 @@ void maildir_quota_set(const char *dir, const char *quota);
 int maildirquota_countfolder(const char *folder);
 int maildirquota_countfile(const char *filename);
 
+void maildir_deliver_quota_warning(const char *dir, const int percent);
+
 #ifdef  __cplusplus
 }
 #endif
