@@ -4,7 +4,7 @@
 */
 
 #if	HAVE_CONFIG_H
-#include	"config.h"
+#include "rfc2045_config.h"
 #endif
 #include	<sys/types.h>
 #include	<sys/stat.h>
@@ -39,7 +39,7 @@
 int gethostname(const char *, size_t);
 #endif
 
-static const char rcsid[]="$Id: makemime.c,v 1.5 2001/05/19 04:01:34 mrsam Exp $";
+static const char rcsid[]="$Id: makemime.c,v 1.11 2003/03/07 00:47:30 mrsam Exp $";
 
 struct arg_list {
 	struct arg_list *next;
@@ -376,6 +376,8 @@ char **argv= *argvp;
 		else
 			m->inputfile2=filename;
 	}
+	else
+		usage();
 
 	/* Handle common options */
 
@@ -621,17 +623,17 @@ struct	mimestruct *m;
 	return (0);
 }
 
-static struct mimestruct *base64m;
-
-static void putbase64(const char *p, size_t l)
+static int encode_outfp(const char *p, size_t n, void *vp)
 {
-	fwrite(p, 1, l, base64m->outputfp);
+	if (fwrite(p, n, 1, *(FILE **)vp) != 1)
+		return -1;
+	return 0;
 }
 
 static void createsimplemime(struct mimestruct *m)
 {
-int	c;
 struct	arg_list *a;
+struct rfc2045_encode_info encode_info;
 
 	/* Determine encoding by reading the file, as follows:
 	**
@@ -642,8 +644,6 @@ struct	arg_list *a;
 
 	if (m->mimeencoding == 0)
 	{
-	int	l=0;
-	int	longline=0;
 	long	orig_pos=ftell(m->inputfp1);
 
 		if (orig_pos == -1)
@@ -652,33 +652,14 @@ struct	arg_list *a;
 			goodexit(m, 1);
 		}
 
-		m->mimeencoding="7bit";
+		m->mimeencoding=rfc2045_encode_autodetect_fp(m->inputfp1, 1);
 
-		while ((c=getc(m->inputfp1)) != EOF)
-		{
-		unsigned char ch= (unsigned char)c;
-
-			if (ch >= 0x80)
-				m->mimeencoding="8bit";
-			if (ch == 0)
-			{
-				m->mimeencoding="base64";
-				longline=0;
-				break;
-			}
-			if (ch == '\n')	l=0;
-			else if (++l > 200)
-				longline=1;
-
-		}
 		if (ferror(m->inputfp1)
 			|| fseek(m->inputfp1, orig_pos, SEEK_SET)<0)
 		{
 			perror("fseek");
 			goodexit(m, 1);
 		}
-		if (longline)
-			m->mimeencoding="quoted-printable";
 
 		if (strcmp(m->mimetype, "auto") == 0)
 			m->mimetype=
@@ -706,63 +687,50 @@ struct	arg_list *a;
 
 	if (m->contentname && *m->contentname)
 	{
-		const char *c;
+		const char *chset=m->textplaincharset ? m->textplaincharset
+			: "iso-8859-1";
+		char *s;
+		const char nameStr[]="name";
+		int len;
 
-		fprintf(m->outputfp, "; name=\"");
-		for (c=m->contentname; *c; c++)
+		rfc2231_attrCreate(nameStr, m->contentname, chset,
+				   NULL,
+				   &len,
+				   "");
+
+		s=malloc(len);
+
+		if (s)
 		{
-			if (*c != '"' && *c != '\\')
-				putc(*c, m->outputfp);
+			rfc2231_attrCreate(nameStr, m->contentname, chset,
+					   s,
+					   &len,
+					   "");
+
+			fprintf(m->outputfp, "; %s", s);
+			free(s);
 		}
-		fprintf(m->outputfp, "\"");
 	}
 
 	fprintf(m->outputfp, "\nContent-Transfer-Encoding: %s\n\n",
 		m->mimeencoding);
 
-	if (strcmp(m->mimeencoding, "quoted-printable") == 0)
+	rfc2045_encode_start(&encode_info, m->mimeencoding,
+			     &encode_outfp,
+			     &m->outputfp);
 	{
-	int	l=0;
-	int	c;
+		char input_buf[BUFSIZ];
+		int n;
 
-		while ((c=getc(m->inputfp1)) != EOF)
+		while ((n=fread(input_buf, 1, sizeof(input_buf),
+				m->inputfp1)) > 0)
 		{
-			if (l > 72)
-			{
-				fprintf(m->outputfp, "=\n");
-				l=0;
-			}
-			if (c == '\n')
-				l=0;
-			else if (c < ' ' || c == '=' || c > 0x7F)
-			{
-				fprintf(m->outputfp, "=%02X",
-					(int)(unsigned char)c);
-				l += 3;
-				continue;
-			}
-			else ++l;
-			putc(c, m->outputfp);
+			if ( rfc2045_encode(&encode_info, input_buf, n))
+				break;
 		}
-		return;
+
+		rfc2045_encode_end(&encode_info);
 	}
-	if (strcmp(m->mimeencoding, "base64") == 0)
-	{
-	char	buf[BUFSIZ];
-	int	l;
-
-		base64m=m;
-		rfc2045_base64encode_start( &putbase64 );
-		while ((l=fread(buf, 1, sizeof(buf), m->inputfp1)) > 0)
-			rfc2045_base64encode(buf, l);
-		rfc2045_base64encode_end();
-		return;
-	}
-
-	/* 7bit or 8bit */
-
-	while ((c=getc(m->inputfp1)) != EOF)
-		putc(c, m->outputfp);
 }
 
 /******************************************************************************
@@ -826,11 +794,11 @@ char	buf[NUMBUFSIZE];
 	do
 	{
 		strcpy(bbuf, "=_");
-		strcat(bbuf, str_size_t(++n, buf));
+		strcat(bbuf, libmail_str_size_t(++n, buf));
 		strcat(bbuf, "_");
-		strcat(bbuf, str_time_t(t, buf));
+		strcat(bbuf, libmail_str_time_t(t, buf));
 		strcat(bbuf, "_");
-		strcat(bbuf, str_pid_t(pid, buf));
+		strcat(bbuf, libmail_str_pid_t(pid, buf));
 	} while (tryboundary(m, f, bbuf));
 	return (bbuf);
 }
@@ -1034,7 +1002,7 @@ FILE	*fp;
 	*/
 
 	buf[0]='&';
-	strcpy(buf+1, str_size_t(pipefd[0], buf2));
+	strcpy(buf+1, libmail_str_size_t(pipefd[0], buf2));
 
 	fp= usescratch ? openfile(buf):openfile_or_pipe(buf, "r");
 	close(pipefd[0]);	/* fd was duped by openfile */
