@@ -1,10 +1,10 @@
 /*
-** Copyright 1998 - 2000 Double Precision, Inc.  See COPYING for
+** Copyright 1998 - 2002 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
 #if	HAVE_CONFIG_H
-#include	"config.h"
+#include "rfc2045_config.h"
 #endif
 #include	<sys/types.h>
 #include	<sys/stat.h>
@@ -26,6 +26,7 @@
 #include	<fcntl.h>
 #include	<signal.h>
 #include	"rfc2045.h"
+#include	"rfc822/rfc822.h"
 #include	"rfc822/rfc2047.h"
 #include	"rfc2045charset.h"
 #if HAVE_UNISTD_H
@@ -41,8 +42,9 @@
 int gethostname(const char *, size_t);
 #endif
 
+extern int rfc2045_in_reformime;
 
-static const char rcsid[]="$Id: reformime.c,v 1.37 2001/05/27 01:43:22 mrsam Exp $";
+static const char rcsid[]="$Id: reformime.c,v 1.45 2003/03/07 00:47:30 mrsam Exp $";
 
 void rfc2045_error(const char *errmsg)
 {
@@ -87,11 +89,15 @@ void usage()
 	fprintf(stderr, "    -X - pipe MIME section to a program.\n");
 	fprintf(stderr, "    -i - show MIME info.\n");
 	fprintf(stderr, "    -s n.n.n.n - specify MIME section.\n");
-	fprintf(stderr, "    -r - rewrite message, filling in missing MIME header.\n");
+	fprintf(stderr, "    -r - rewrite message, filling in missing MIME headers.\n");
 	fprintf(stderr, "    -r7 - also convert 8bit/raw encoding to quoted-printable, if possible.\n");
 	fprintf(stderr, "    -r8 - also convert quoted-printable encoding to 8bit, if possible.\n");
-	fprintf(stderr, "    -c charset - when rewriting, specify default charset.\n");
+	fprintf(stderr, "    -c charset - default charset for rewriting, -o, and -O.\n");
 	fprintf(stderr, "    -m [file] [file]... - create a MIME message digest.\n");
+	fprintf(stderr, "    -h \"header\" - decode RFC 2047-encoded header.\n");
+	fprintf(stderr, "    -o \"header\" - encode unstructured header using RFC 2047.\n");
+	fprintf(stderr, "    -O \"header\" - encode address list header using RFC 2047.\n");
+
 	exit(1);
 }
 
@@ -102,9 +108,9 @@ static unsigned counter=0;
 time_t	t;
 char	*p;
 
-	str_pid_t(getpid(), pidbuf);
+	libmail_str_pid_t(getpid(), pidbuf);
 	time(&t);
-	str_time_t(t, timebuf);
+	libmail_str_time_t(t, timebuf);
 	hostnamebuf[sizeof(hostnamebuf)-1]=0;
 	if (gethostname(hostnamebuf, sizeof(hostnamebuf)))
 		hostnamebuf[0]=0;
@@ -121,6 +127,7 @@ struct rfc2045 *read_message()
 char	buf[BUFSIZ];
 struct	rfc2045 *p=rfc2045_alloc_ac();
 FILE	*tempfp=0;
+int	l;
 
 	if (fseek(stdin, 0L, SEEK_END) < 0 ||
 		fseek(stdin, 0L, SEEK_SET) < 0)	/* Pipe, save to temp file */
@@ -128,9 +135,8 @@ FILE	*tempfp=0;
 		tempfp=tmpfile();
 	}
 
-	while (fgets(buf, sizeof(buf), stdin))
+	while ((l=fread(buf, 1, sizeof(buf), stdin)) > 0)
 	{
-	int l=strlen(buf);
 
 		rfc2045_parse(p, buf, l);
 		if (tempfp && fwrite(buf, l, 1, tempfp) != 1)
@@ -139,6 +145,7 @@ FILE	*tempfp=0;
 			exit(1);
 		}
 	}
+	rfc2045_parse_partial(p);
 
 	if (tempfp)	/* Replace stdin */
 	{
@@ -164,61 +171,53 @@ static void do_print_info(struct rfc2045 *s)
 {
 const char *content_type, *transfer_encoding, *charset;
 off_t start, end, body;
-const char *disposition;
-const char *disposition_name;
-const char *disposition_filename;
-const char *content_name;
+char *content_name;
 off_t nlines, nbodylines;
 const char *p;
 
+char *disposition_name, *disposition_filename;
+
 	rfc2045_mimeinfo(s, &content_type, &transfer_encoding, &charset);
 	rfc2045_mimepos(s, &start, &end, &body, &nlines, &nbodylines);
-	rfc2045_dispositioninfo(s, &disposition, &disposition_name,
-		&disposition_filename);
-	content_name=rfc2045_contentname(s);
-	printf("content-type: %s\n", content_type);
-	if (content_name)
-	{
-	char *s=rfc2047_decode_simple(content_name);
 
-		if (!s)
-		{
-			perror("rfc2047_decode_simple");
-			exit(1);
-		}
-		printf("content-name: %s\n", s);
-		free(s);
+	if (rfc2231_udecodeType(s, "name", NULL, &content_name) < 0)
+	{
+		perror("malloc");
+		exit(1);
 	}
 
+	printf("content-type: %s\n", content_type);
+	if (*content_name)
+	{
+		printf("content-name: %s\n", content_name);
+	}
+	free(content_name);
+
 	printf("content-transfer-encoding: %s\n", transfer_encoding);
 	printf("charset: %s\n", charset);
-	if (disposition && *disposition)
-		printf("content-disposition: %s\n", disposition);
-	if (disposition_name && *disposition_name)
-	{
-	char *s=rfc2047_decode_simple(disposition_name);
+	if (s->content_disposition && *s->content_disposition)
+		printf("content-disposition: %s\n", s->content_disposition);
 
-		if (!s)
-		{
-			perror("rfc2047_decode_simple");
-			exit(1);
-		}
-		printf("content-disposition-name: %s\n", s);
-		free(s);
+	if (rfc2231_udecodeDisposition(s, "name", NULL, &disposition_name) < 0
+	    ||
+	    rfc2231_udecodeDisposition(s, "filename", NULL,
+				       &disposition_filename) < 0)
+	{
+		perror("malloc");
+		exit(1);
 	}
 
-	if (disposition_filename && *disposition_filename)
-	{
-	char *s=rfc2047_decode_simple(disposition_filename);
+	if (*disposition_name)
+		printf("content-disposition-name: %s\n", disposition_name);
 
-		if (!s)
-		{
-			perror("rfc2047_decode_simple");
-			exit(1);
-		}
-		printf("content-disposition-filename: %s\n", s);
-		free(s);
+	free(disposition_name);
+
+	if (*disposition_filename)
+	{
+		printf("content-disposition-filename: %s\n",
+		       disposition_filename);
 	}
+	free(disposition_filename);
 
 	if (*(p=rfc2045_content_id(s)))
 		printf("content-id: <%s>\n", p);
@@ -332,25 +331,35 @@ void rewrite(struct rfc2045 *p, int rwmode)
 static char *get_suitable_filename(struct rfc2045 *r, const char *pfix,
 	int ignore_filename)
 {
-const char *disposition_s;
-const char *disposition_name_s;
-const char *disposition_filename_s;
+char *disposition_name;
+char *disposition_filename;
 char	*filename_buf;
-const char *content_name_s;
+char *content_name;
 char	*p, *q;
 char	*dyn_disp_name=0;
 
-	rfc2045_dispositioninfo(r, &disposition_s, &disposition_name_s,
-		&disposition_filename_s);
+const char *disposition_filename_s;
 
-	content_name_s=rfc2045_contentname(r);
+	if (rfc2231_udecodeDisposition(r, "name", NULL, &disposition_name) < 0
+	    ||
+	    rfc2231_udecodeDisposition(r, "filename", NULL,
+				       &disposition_filename) < 0
+	    ||
+	    rfc2231_udecodeType(r, "name", NULL,
+				&content_name) < 0)
+	{
+		perror("malloc");
+		exit(1);
+	}
+
+	disposition_filename_s=disposition_filename;
 
 	if (!disposition_filename_s || !*disposition_filename_s)
-		disposition_filename_s=disposition_name_s;
+		disposition_filename_s=disposition_name;
 	if (!disposition_filename_s || !*disposition_filename_s)
-		disposition_filename_s=content_name_s;
+		disposition_filename_s=content_name;
 
-	filename_buf=rfc2047_decode_simple(disposition_filename_s);
+	filename_buf=strdup(disposition_filename_s);
 
 	if (!filename_buf)
 	{
@@ -358,6 +367,10 @@ char	*dyn_disp_name=0;
 		exit(1);
 	}
 
+	free(content_name);
+	free(disposition_name);
+	free(disposition_filename);
+
 	if (strlen(filename_buf) > 32)
 	{
 		p=filename_buf;
@@ -390,7 +403,7 @@ char	*dyn_disp_name=0;
 	{
 	char	numbuf[NUMBUFSIZE];
 	static size_t counter=0;
-	const char *p=str_size_t(++counter, numbuf);
+	const char *p=libmail_str_size_t(++counter, numbuf);
 
 		dyn_disp_name=malloc(strlen(disposition_filename_s)
 			+ strlen(p)+2);
@@ -895,6 +908,10 @@ char	optc;
 char	*optarg;
 char	*mimesection=0;
 int	doinfo=0, dodecode=0, dorewrite=0, dodsn=0, domimedigest=0;
+int	dodecodehdr=0, doencodemime=0, doencodemimehdr=0;
+
+char	*decode_header="";
+ char	*mimecharset="iso-8859-1";
 struct	rfc2045 *p;
 int	rwmode=0;
 int	dovalidate=0;
@@ -906,6 +923,8 @@ int rc=0;
 	setlocale(LC_ALL, "C");
 #endif
 
+	rfc2045_in_reformime=1;
+
 	for (argn=1; argn<argc; )
 	{
 		if (argv[argn][0] != '-')	break;
@@ -918,7 +937,10 @@ int rc=0;
 			if (!optarg && argn < argc)
 				optarg=argv[argn++];
 			if (optarg && *optarg)
+			{
+				mimecharset=optarg;
 				rfc2045_setdefaultcharset(optarg);
+			}
 			break;
 
 		case 's':
@@ -962,6 +984,34 @@ int rc=0;
 		case 'V':
 			dovalidate=1;
 			break;
+		case 'h':
+			if (!optarg && argn < argc)
+				optarg=argv[argn++];
+			if (optarg)
+			{
+				decode_header=optarg;
+			}
+			dodecodehdr=1;
+			break;
+		case 'o':
+			if (!optarg && argn < argc)
+				optarg=argv[argn++];
+			if (optarg)
+			{
+				decode_header=optarg;
+			}
+			doencodemime=1;
+			break;
+		case 'O':
+			if (!optarg && argn < argc)
+				optarg=argv[argn++];
+			if (optarg)
+			{
+				decode_header=optarg;
+			}
+			doencodemimehdr=1;
+			break;
+
 		default:
 			usage();
 		}
@@ -972,6 +1022,44 @@ int rc=0;
 		mimedigest(argc-argn, argv+argn);
 		return (0);
 	}
+	else if (dodecodehdr)
+	{
+		char *s=rfc2047_decode_simple(decode_header);
+
+		if (s)
+		{
+			printf("%s\n", s);
+			free(s);
+		}
+		return (0);
+	}
+	if (doencodemime)
+	{
+		char *s=rfc2047_encode_str(decode_header, mimecharset);
+
+		if (s)
+		{
+			printf("%s\n", s);
+			free(s);
+		}
+		return (0);
+	}
+	if (doencodemimehdr)
+	{
+		struct rfc822t *t=rfc822t_alloc_new(decode_header, NULL, NULL);
+		struct rfc822a *a=t ? rfc822a_alloc(t):NULL;
+		char *s;
+
+		if (a && (s=rfc2047_encode_header(a, mimecharset)) != NULL)
+		{
+			printf("%s\n", s);
+			free(s);
+		}
+
+		if (a) rfc822a_free(a);
+		if (t) rfc822t_free(t);
+		return (0);
+	}
 
 	p=read_message();
 
