@@ -1,13 +1,13 @@
 /*
-** Copyright 1998 - 2002 Double Precision, Inc.  See COPYING for
+** Copyright 1998 - 2003 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
 /*
-** $Id: rfc2045.c,v 1.30 2002/03/19 01:29:18 mrsam Exp $
+** $Id: rfc2045.c,v 1.34 2003/04/04 04:38:23 mrsam Exp $
 */
 #if    HAVE_CONFIG_H
-#include       "config.h"
+#include       "rfc2045_config.h"
 #endif
 #include       <stdlib.h>
 #include       <stdio.h>
@@ -266,9 +266,11 @@ static void update_counts(struct rfc2045 *p, size_t newcnt, size_t newendcnt,
 
 static void doline(struct rfc2045 *);
 
+void rfc2045_parse_partial(struct rfc2045 *h);
+
 void rfc2045_parse(struct rfc2045 *h, const char *buf, size_t s)
 {
-size_t	l;
+	size_t	l;
 
 	while (s)
 	{
@@ -287,12 +289,18 @@ size_t	l;
 		s -= l;
 	}
 
+	if (h->workbuflen > 1024)
+		rfc2045_parse_partial(h);
+}
+
+void rfc2045_parse_partial(struct rfc2045 *h)
+{
 	/*
 	** Our buffer's getting pretty big.  Let's see if we can
 	** partially handle it.
 	*/
 
-	if (h->workbuflen > 512)
+	if (h->workbuflen > 0)
 	{
 	struct	rfc2045 *p;
 	int	l, i;
@@ -1166,6 +1174,7 @@ int	i;
 
 	s=paste_tokens(t, 2, t->ntokens-2);
 	set_string(&p->content_location, s);
+	free(s);
 }
 
 /* -------------------- */
@@ -1226,26 +1235,6 @@ int rfc2045_isflowed(const struct rfc2045 *p)
 	return (cb && strcmp(cb, "flowed") == 0);
 }
 
-void rfc2045_dispositioninfo(const struct rfc2045 *p,
-	const char **disposition_s,
-	const char **disposition_name_s,
-	const char **disposition_filename_s)
-{
-	*disposition_s=p->content_disposition;
-	*disposition_name_s=rfc2045_getattr(p->content_disposition_attr,
-				"name");
-	*disposition_filename_s=rfc2045_getattr(p->content_disposition_attr,
-				"filename");
-}
-
-const char *rfc2045_contentname(const struct rfc2045 *p)
-{
-const	char *q=rfc2045_getattr(p->content_type_attr, "name");
-
-	if (!q)	q="";
-	return (q);
-}
-
 const char *rfc2045_content_id(const struct rfc2045 *p)
 {
 	return (p->content_id ? p->content_id:"");
