@@ -3,7 +3,7 @@
 ** distribution information.
 */
 
-#include	"config.h"
+#include "rfc2045_config.h"
 #include	"rfc2045.h"
 #include	"rfc2646.h"
 #include	"rfc822/rfc2047.h"
@@ -15,7 +15,7 @@
 #include	<string.h>
 #include	<ctype.h>
 
-static const char rcsid[]="$Id: rfc2045reply.c,v 1.7 2001/01/21 17:34:36 mrsam Exp $";
+static const char rcsid[]="$Id: rfc2045reply.c,v 1.9 2003/03/07 00:47:31 mrsam Exp $";
 
 extern void rfc2045_enomem();
 
@@ -382,6 +382,11 @@ static int mkforward(struct rfc2045_mkreplyinfo *ri)
 			rfc2646_free(parser);
 			rfc2646fwd_free(p);
 		}
+		else if (strcmp(ri->replymode, "forwardatt") == 0)
+		{
+			if (lseek(ri->fd, start_pos, SEEK_SET) == 0)
+				forwardbody(ri, end_pos - start_pos);
+		}
 		else	(*ri->decodesectionfunc)(ri->fd, ri->rfc2045partp,
 						 ri->charset,
 						 &write_wrap_func, ri);
