@@ -1,5 +1,5 @@
 /*
-** $Id: rfc822.h,v 1.20 2000/10/22 05:23:50 mrsam Exp $
+** $Id: rfc822.h,v 1.22 2003/03/07 01:49:32 mrsam Exp $
 */
 #ifndef	rfc822_h
 #define	rfc822_h
@@ -10,7 +10,7 @@
 */
 
 #if	HAVE_CONFIG_H
-#include	"config.h"
+#include	"../rfc822/config.h"	/* VPATH build */
 #endif
 
 #include	<time.h>
@@ -86,6 +86,11 @@ struct rfc822t {
 
 struct rfc822t *rfc822t_alloc(const char *p,
 	void (*err_func)(const char *, int));	/* Parse addresses */
+
+struct rfc822t *rfc822t_alloc_new(const char *p,
+	void (*err_func)(const char *, int, void *), void *);
+	/* Parse addresses */
+
 void rfc822t_free(struct rfc822t *);		/* Free rfc822 structure */
 
 void rfc822tok_print(const struct rfc822token *, void (*)(char, void *), void *);
