@@ -2,11 +2,11 @@
 #define	userdb_h
 
 /*
-** Copyright 1998 - 1999 Double Precision, Inc.
+** Copyright 1998 - 2001 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
-static const char userdb_rcsid_h[]="$Id: userdb.h,v 1.3 1999/12/15 05:22:48 mrsam Exp $";
+static const char userdb_rcsid_h[]="$Id: userdb.h,v 1.5 2001/06/17 18:54:54 mrsam Exp $";
 
 #include	<sys/types.h>
 
@@ -48,11 +48,14 @@ struct userdbs {
 	char *udb_quota;	/* Maildir quota */
 	uid_t	udb_uid;
 	gid_t	udb_gid;
+
+	char *udb_source;	/* Non-blank - source file in userdb dir */
 	} ;
 
 struct userdbs *userdb_creates(const char *);
 struct userdbs *userdb_createsuid(uid_t);
 void	userdb_frees(struct userdbs *);
+char *userdb_mkmd5pw(const char *);
 
 #ifdef	__cplusplus
 } ;
